﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("DTLZ2", "Testfunction as defined as DTLZ2 in http://repository.ias.ac.in/81671/ [30.11.15]")]
  [StorableClass]
  public class DTLZ2 : MultiObjectiveTestFunction {

    private int actualSolutionSize = 2;
    public override int ActualSolutionSize {
      get {
        return actualSolutionSize;
      }

      set {
        actualSolutionSize = value;
      }
    }

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { 0, 1 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[actualSolutionSize];
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {

        return int.MaxValue;
      }
    }

    public override int MinimumProblemSize {
      get {
        return Math.Max(2, ActualSolutionSize);
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    public override RealVector[] OptimalParetoFront {
      get {
        throw new NotImplementedException();
      }
    }

    public override RealVector ReferencePoint {
      get { return new RealVector(new double[] { 11.0, 11.0 }); }
    }

    public override double BestKnownHypervolume {
      get { return 121.0 - 1.0 / 4.0 *Math.PI; }
    }

    [StorableConstructor]
    protected DTLZ2(bool deserializing) : base(deserializing) { }
    protected DTLZ2(DTLZ2 original, Cloner cloner) : base(original, cloner) { }
    public DTLZ2() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new DTLZ2(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      return Evaluate(r, actualSolutionSize);
    }

    private double[] Evaluate(RealVector r, int objectives) {
      if (r.Length < objectives) {
        throw new Exception("The dimensionality of the problem(ProblemSize) must be larger or equal than the dimensionality of the solution(SolutionSize) ");
      }

      double[] res = new double[objectives];

      //calculate g(Xm)
      double g = 0;
      for (int i = objectives; i < r.Length; i++) {
        double d = r[i] - 0.5;
        g += d * d;
      }

      //calculating f0...fM-1
      for (int i = 0; i < objectives; i++) {
        double f = i == 0 ? 1 : (Math.Sin(r[objectives - i - 1] * Math.PI / 2)) * (1 + g);
        for (int j = 0; j < objectives - i - 1; j++) {
          f *= Math.Cos(r[j] * Math.PI / 2);
        }
        res[i] = f;
      }
      return res;
    }
  }
}
