﻿using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.MultiObjectiveTestFunctions {
  [Item("ZDT1", "ZDT1 function as defined in http://www.tik.ee.ethz.ch/sop/download/supplementary/testproblems/ [30.11.2015]")]
  [StorableClass]
  public class ZDT1 : MultiObjectiveTestFunction {

    public override DoubleMatrix Bounds {
      get {
        return new DoubleMatrix(new double[,] { { 0, 1 } });
      }
    }

    public override bool[] Maximization {
      get {
        return new bool[] { false, false };
      }
    }

    public override int MaximumProblemSize {
      get {
        return int.MaxValue;
      }
    }

    public override int MaximumSolutionSize {
      get {
        return 2;
      }
    }

    public override int MinimumProblemSize {
      get {
        return 1;
      }
    }

    public override int MinimumSolutionSize {
      get {
        return 2;
      }
    }

    public override int ActualSolutionSize {
      get {
        return 2;
      }

      set {
      }
    }

    public override RealVector[] OptimalParetoFront {
      get {
        throw new NotImplementedException();
      }
    }

    public override RealVector ReferencePoint {
      get { return new RealVector(new double[]{ 11.0, 11.0}); }
    }

    public override double BestKnownHypervolume {
      get { return 120 + 2.0 / 3; }
    }

    [StorableConstructor]
    protected ZDT1(bool deserializing) : base(deserializing) { }
    protected ZDT1(ZDT1 original, Cloner cloner) : base(original, cloner) { }
    public ZDT1() : base() { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ZDT1(this, cloner);
    }



    public override double[] Evaluate(RealVector r) {
      double g = 0;
      for (int i = 1; i < r.Length; i++) g += r[i];
      g = 1.0 + 9.0 * g / (r.Length - 1);
      double f1 = g * (1.0 - Math.Sqrt(r[0] / g));
      return new double[] { r[0], f1};
    }
  }
}
